/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __ALGORITHMS_H
#define __ALGORITHMS_H

#include "Modelib.h"
#include <iostream>

namespace Modelib {

//! for_all Elmt in Container
//!      Prop(Elmt,...,Elmt+n)
//!
//!  typiquement:
//!              Container = FloatVarArray
//!              Elmt      = FloatVar
//!              Prop      = functor
//!
//!
template<typename Elmt, class Container, class Prop>
struct for_all_var
{
  for_all_var() {};
  void operator()(const Container& _c) const
  {
       const unsigned len = _c.Size();
       Prop MaProp;
       unsigned i=0;
       for (;i<len;++i)
       {
         Elmt *t = reinterpret_cast<Elmt *>(&_c[i]);
         MaProp(*t);
       }
  }
};

template<typename Elmt, class Container, class Prop>
struct for_all_num
{
  for_all_num() {};
  void operator()(const Container& _c) const
  {
       const unsigned len = _c.Size();
       Prop MaProp;
       unsigned i=0;
       for (;i<len;++i)
       {
         Elmt t(static_cast<int>(_c[i]));
         //float val = _c[i];
         //t = val;
         MaProp(t);
       }
  }
};

/** Tant que PropVerify()(Elmt) est vrifie, on applique PropOper()(Elmt)
  */
template<typename Elmt, class Container, class PropOper, class PropVerify>
struct while_prop
{
  while_prop() {};

  void operator()(const Container& _c) const
  {
       const unsigned len = _c.Size();
       Elmt *_e = 0;
       PropOper   MaProp;
       PropVerify Condition;
       unsigned i=0;
       for (;i<len;++i)
       {
         _e = reinterpret_cast<Elmt*>(&_c[i]);
         if (Condition(*_e) == true)
           MaProp(*_e);
       }
  }
};

/** Permet de faire la somme de tous les Elmt du Container
  */
template<typename Elmt, class Container>
struct sum
{
  sum(){};
  float operator()(const Container& _c) const
  {
       const unsigned len = _c.Size();
       Elmt *_e = 0;
       float _res=0,t=0;
       unsigned i=0;
       for (;i<len;++i)
       {
         _e = reinterpret_cast<Elmt*>(&_c[i]);
         if ((t=_e->GetValue()) != 0)
           _res = t + _res;
       }
       return _res;
  }
};


/** Rentre un lment boolen
  */
template<typename Elmt>
struct BoolIT
{
  BoolIT(){};
  void operator()(Elmt& _e) const
  {
    if (_e.GetValue() <= 0) _e.SetValue(0);
    else                    _e.SetValue(1);
  }
};

/** Retourne true si la valeur est non nulle
  */
template<typename Elmt>
struct NotNull
{
  NotNull(){};
  bool operator()(Elmt& _e) const
  {
    if (_e.GetValue() != 0) return true;
    else                    return false ;
  }
};

/** Affichage d'un lment
  */
template<typename Elmt>
struct Affiche
{
  Affiche() {};
  void operator()(Elmt& _e)
  {
    std::cout << _e.GetValue() << ',' ;
  }
};

template<typename Elmt, class Container>
Elmt Min(Container& _c)
{
  unsigned i=1;
  Elmt min=_c[0];
  while (_c[i])
  {
    if (_c[i] < min.GetValue())
       min = _c[i];
    i++;
  }
  return min;
}

template<typename Elmt, class Container>
Elmt Max(Container& _c)
{
  unsigned i=1;
  Elmt max=_c[0];
  while (_c[i])
  {
    if (_c[i] > max.GetValue())
       max = _c[i];
    i++;
  }
  return max;
}


template<class Container>
struct OBJECTIVE_SUM
{
  Expr operator()(Container& _c)
  {
    return Sum(_c);
  }
};

template<class Container_VAR, class Container_NUM>
struct OBJECTIVE_PROD // on fait le produit scalaire
{
  Expr operator()(Container_VAR& _c_var, Container_NUM& _c_num)
  {
    return Expr(_c_var * _c_num);
  }
};



template<class Container_to, class Container_from>
void Set(Container_to& _to, Container_from& _from)
{
  unsigned i=0;
  while (_from[i])
  {
    _to[i] = _from[i];
    i++;
  }
}

}

#endif

/*

  exemple d'utilisation:
      
      FloatVarArray t(m,NOMBRE_DE_VAR,-Infinity,Infinity,"vect");
      
      // Applique PropType() a tous les lments de t              
      for_all<FloatVar,FloatVarArray,PropType<FloatVar> >()(t);

      // Applique PropType() a tous les lments de t tant que ceux-ci
      // vrifient la condition Superieur()
      while_prop<FloatVar,FloatVarArray,PropType<FloatVar>,Superieur<FloatVar> >()(t);
      
      // Calculer la "somme" de t
      FloatVar somme = sum<FloatVar,FloatVarArray>()(t);
*/


