/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __CONSTRAINT_H
#define __CONSTRAINT_H

#include "Constant.h"
#include "LagrangianParam.h"
#include <string>

namespace Modelib {

class Model;
class Expr;

///Classe representant les Contraintes
/**Cette classe sert d'une part a la gestion de contraintes
  particuliaires  et a la relaxation lagrangienne
  Elle permet entre autres l'ajout d'une expression
  ou la modification du second membre pour la contrainte cible
 */
class Constraint
{
    Model * model;
    VarId id;

    public:
       /**Contructeur par defaut*/
       Constraint():model(0),id(0) {};

       Constraint(Model & _m,VarId _id);
       Constraint( const Model & _m,VarId _id);

       /**Constructeur de copie*/
       Constraint( const Constraint & _c)
       {
       	 model = _c.model;
       	 id = _c.id;
       }
       
       Constraint operator= ( const Constraint & _c)
       {
      	 model = _c.model;
       	 id = _c.id;
       	 return *this;
       }

        /**Geter de l'identifiant de contrainte
          @return L'identifiant de contrainte sous le format VarId.
          */
        VarId GetId() const { return id;}

        /**Geter du modle
            @return Un pointeur sur le Model o est dfinie la contrainte.
          */
        Model * GetModel() const { return model;}

        bool Relax( float lagrangianValue = -Infinity);

        LagrangianParam GetLagrangianParam() const;

        void Rename( const std::string & name );

        void AddExpr(const Expr &);

        Expr GetExpr(bool secondMember = false) const;

        Expr GetRelaxedExpr(bool secondMember = false) const;

        void SetSecondMember(float);

        float GetSecondMember() const;

        float GetValue() const;

       ~Constraint() {}
};

inline bool operator==( const Constraint & c1, const Constraint & c2)
{
    return c1.GetModel() == c2.GetModel() && c1.GetId() == c2.GetId();
}

#ifdef COMPILE_FOR_PYTHON
	void export_Constraint();
#endif

}

#endif
