/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

///: Chargement/Ecriture dans un fichier LP
#ifndef __FICHIER_MPS_H
#define __FICHIER_MPS_H


#include "FichierProbleme.h"

namespace Modelib {

/** FichierMPS est une spcification d'un fichier contenant
    un probleme.
    Cette classe permet de charger des fichiers de type MPS et
    hrite donc de FichierProbleme.
*/
class FichierMPS : public FichierProbleme
{
protected:
  std::map< std::string, plGlobalNode * > constraintMap;
  std::string objectifName;
  bool integerMarker;

  void ParseROWS(const std::string& );
  void ParseCOLUMNS(const std::string& );
  void ParseRHS(const std::string& );
  void ParseBOUNDS(const std::string& );

public:
  //! Constructeur par defaut (et unique constructeur)
  //! On ne devrait pas avoir besoin de constructeur par copie car cette classe
  //! n'a d'instances que temporaires !
  FichierMPS();

  //! Ouverture et lecture
  bool Open(const std::string& );

  //! Destructeur
  ~FichierMPS()
  {
    //std::cout << "Destructeur de la spcialisation: ~FichierMPS()"  << std::endl;
  };
};

}

#endif
///~
