/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following
	conditions are met:

		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef __GLPK_SOLVER_H
#define __GLPK_SOLVER_H

#include "Solver.h"
#include "Solution.h"

#include <iostream>
#include <string>


namespace Modelib {

class GlpkSolver : public Solver
{
	std::string tempName;

  // Quelques fonctions lies au solveur GLPK
  private:

	//! Palier au probleme de GLPK pour les lignes de plus de 256 caractres
	std::ostream& EcrireBonneLargeur(std::ostream&, const std::string&);

	//! Crer un fichier temporaire pour GLPK
	void CreerFichierTemporaire(const std::string&);

	//! Appeler la rsolution (appel systme)
	void Resoudre(const std::string& solName);
	
	//! Interpreter la solution
	void Interprete(const std::string& , Solution& );


  // L'interface de notre solveur
  public:
	//! Constructeur
	GlpkSolver();
	//! Destructeur
	~GlpkSolver();

	//! Appel du solveur
	float Solve(Solution&, const std::string&);
};

}

#endif
