/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifdef __MATRICE_CREUSE_H
      #define __export_template template
#else
      #include "MatriceCreuse.h"

      #include <map>
      #include <iostream>
      #include <string>
      #include <list>
      #include <fstream>
      #include <sstream>

      #define __export_template export template
#endif

using namespace std;

/**
  Crer et charger une matrice depuis un fichier contenant
  une liste de triplet ligne colonne value
  @param nom l'adresse du fichier
  @param defaultT (facultatif) l'element par defaut
 */
__export_template<typename T>
MatriceCreuse<T>::MatriceCreuse(const string& nom, T _defaultT)
  : nbCols(0), nbLignes(0), defaultT( _defaultT )
{
    // ouvrir le fichier nom
    ifstream      ifile(nom.c_str());
    
    if(!ifile.is_open())
        throw runtime_error("MatriceCreuse<T>::MatriceCreuse(const string& nom, T _defaultT) : IO Error");

    // boucler sur les lignes
    unsigned _i=0,_j=0,_im=0,_jm=0;
    T _val = defaultT;

    while (ifile >> _i >> _j >> _val)
    {
      Id S(_i,_j);
      if (!(_val == defaultT))
      {
        matrix[S] = _val;
      }

      if (_i > _im) _im = _i;
      if (_j > _jm) _jm = _j;

    }
    ifile.close();

    nbCols = _im;
    nbLignes = _jm;
}

/**
  Ouverture d'un fichier puis stockage dans la matrice
  Le fichier doit tre une suite de triplet ligne colonne value
  @param nom l'adresse du fichier
  @return vrai si l'opration est un succs
 */
__export_template<typename T>
bool MatriceCreuse<T>::Open(const string& nom)
{
  // ouvrir le fichier nom
  ifstream ifile(nom.c_str());
  
  if(!ifile.is_open())
     throw runtime_error("bool MatriceCreuse<T>::Open(const string& nom) : I/O Error");


  // boucler sur les lignes
  unsigned _i=0,_j=0,_im=0,_jm=0;
  T _val = defaultT;

  while (ifile >> _i >> _j >> _val)
  {
    Id S(_i,_j);
    if (!(_val == defaultT))
    {
      matrix[S] = _val;
    }
    
    if (_i > _im) _im = _i;
    if (_j > _jm) _jm = _j;

  }
  ifile.close();

  nbCols   = _im;
  nbLignes = _jm;
  return true;
} ///~


/**
  surcharge de l'oprateur () de la classe matrice
  @param i colonne
  @param j ligne
  @return lement se trouvant au coordonnes (i,j)
 */__export_template<typename T>
T MatriceCreuse<T>::operator() (unsigned i, unsigned j)
{
    const Id S(i,j);
    typename map<Id,T,Compare<T> >::const_iterator iter = matrix.find(S);
    if (iter != matrix.end())
            return iter->second;
    return defaultT;
}

/**
  surcharge de l'oprateur () de la classe matrice dans le cas d'une matrice constante
  @param i colonne
  @param j ligne
  @return lement se trouvant au coordonnes (i,j)
 */
__export_template<typename T>
T MatriceCreuse<T>::operator() (unsigned i, unsigned j) const
{
    const Id S(i,j);
    typename map<Id,T,Compare<T> >::const_iterator iter = matrix.find(S);
    if (iter != matrix.end())
            return iter->second;
    return defaultT;
}

/**
  Geter de coefficient de matrice
  @param i colonne
  @param j ligne
  @return lement se trouvant au coordonnes (i,j)
 */
__export_template<typename T>
T MatriceCreuse<T>::Get(unsigned i, unsigned j) const
{
    const Id S(i,j);
    typename map<Id,T,Compare<T> >::const_iterator iter = matrix.find(S);
    if (iter != matrix.end())
            return iter->second;
    return defaultT;
}


// --------------- Hors classe -------------
/*
__export_template<typename T>
ostream& operator<<(ostream& o, const MatriceCreuse<T>& mat)
{
  unsigned i;
  for (unsigned j=0;j<mat.GetnbLignes();j++)
  {
    for (i=0;i<mat.GetnbCols();i++)
    {
        o << mat(i,j);
        o << ',';
    }
    o << endl;
  }
  return o;
}
*/
