/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __NOM_H
#define __NOM_H

#include <string>
#include <sstream>

namespace Modelib {

/**
  Construction des noms d'une variable
  On a surcharg les constructeurs afin de pouvoir construire de manire tres
  varies les noms des variables.
  
  ex:
    Qq soit le type de i,j:
      Nom nom("X",i,j);
      nom.GetNom();

*/
class Nom
{
  std::string nom;

public:

  Nom(){};
  Nom(const std::string& _var) : nom(_var) {};
  Nom(const std::string& _var, const int _i     ) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };
  Nom(const std::string& _var, const unsigned _i) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };
  Nom(const std::string& _var, const long _i    ) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };
  Nom(const std::string& _var, const float _i   ) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };
  Nom(const std::string& _var, const double _i  ) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };
  Nom(const std::string& _var, const char _i    ) : nom(_var) { std::ostringstream s; s << _i; nom += std::string(s.str()); };

  Nom(const std::string& _var, const int _i     , const int _j     ) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const unsigned _i, const unsigned _j) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const long _i    , const long _j    ) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const float _i   , const float _j   ) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const double _i  , const double _j  ) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const char _i    , const char _j    ) : nom(_var) {  std::ostringstream s,ss; s << _i; nom += std::string(s.str()); s.clear(); ss << _j; nom += std::string(ss.str()); };
  
  Nom(const std::string& _var, const int _i     , const std::string& _c, const int _j     ) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const unsigned _i, const std::string& _c, const unsigned _j) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const long _i    , const std::string& _c, const long _j    ) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const float _i   , const std::string& _c, const float _j   ) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const double _i  , const std::string& _c, const double _j  ) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  Nom(const std::string& _var, const char _i    , const std::string& _c, const char _j    ) : nom(_var) { std::ostringstream s,ss; s << _i; nom += std::string(s.str()); nom += _c; s.clear(); ss << _j; nom += std::string(ss.str()); };
  
  Nom& operator<<(const std::string& _str)
  {
    nom += _str;
    return *this;
  }
  Nom& operator<<(const int _i     ) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}
  Nom& operator<<(const unsigned _i) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}
  Nom& operator<<(const long _i    ) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}
  Nom& operator<<(const double _i  ) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}
  Nom& operator<<(const float _i   ) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}
  Nom& operator<<(const char _i    ) { std::ostringstream s; s << _i; nom += std::string(s.str()); return *this;}

  std::string operator()() const
  {
    return nom;
  }

  Nom( const Nom & _nom) { *this = _nom; }
  
  Nom & operator=( const Nom & _nom )
  {
  	 if( this != &_nom)
  	     nom = _nom.nom;
  	 return *this;
  }

  ~Nom(){};

  std::string GetNom() const {return nom; }

  void        Clear()        {nom.clear();}
  void        SetNom(const std::string& _nom) {nom = _nom;}

};

}

#endif
