/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __W_PNG_H
#define __W_PNG_H

#include <string>
#include <png.h>
#include <iostream>

namespace Modelib {

class PNGImage
{


public:
  std::string filename;
  png_structp png_ptr;
  png_infop   info_ptr;
  unsigned    width, height;
  png_byte  **image;
  png_byte  **row_pointers;
  FILE       *fp;
  int bytes_per_pixel;
  
public:
  PNGImage() {  std::cerr.rdbuf(0);};

  // appel pour charger une image..
  PNGImage(int _width, int _height) : fp(NULL), bytes_per_pixel(3)
  {

    std::cerr << "Void cstor";
    width = _width;
    height = _height;
    row_pointers = new png_byte*[width*bytes_per_pixel];
    std::cerr.rdbuf(0);
  }

  // constructeur que l'on va appeler pour creer une image
  PNGImage(const std::string& _filename, int _width, int _height)
    : filename(_filename),
      png_ptr(NULL), info_ptr(NULL),
      width(_width), height(_height),
      fp(NULL),
      bytes_per_pixel(3)
  {

    // allocation de ** image
    // [height][width*bytes_per_pixel]
    std::cerr.rdbuf(0);

    image = new png_byte* [height];
    for (unsigned i=0;i<height;i++)
        image[i] = new png_byte [width*bytes_per_pixel];

    row_pointers = new png_byte*[width*bytes_per_pixel];
  }



  ~PNGImage();

  bool openFileRead();
  bool openFileWrite();
  bool isPNG(int bytesToCheck);
  bool initReadStructs();
  bool initWriteStructs();
  bool writeHeader();

  png_byte** writeImage_Start();
  bool       writeImage_End();

  bool writeEnd();

};

}

#endif
