#!/usr/bin/env python
"""
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or
	without modification, are permitted provided that the following
	conditions are met:
	
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
	
		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.
	
		* Neither the name of Quentin Lequy or Romain Gaucher nor the names
		  of its contributors may be used to endorse or promote products
		  derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



	Structure de l'arborescence de Modelib
	- modelib
		- bin
		- docs
		- include
		- sources
	+ install.py
	+ licences etc.

	For each file in sources, check the include !
	If it's okay, compile with some options.... ask each time for
	the option
"""
import os, sys
import re

cpp = re.compile("(.+).cpp$", re.I)
hpp = re.compile("(.+).h|(.+).cxx", re.I)

comp_py = re.compile("py_(.+)", re.I)
comp_o = re.compile("(.+).o$", re.I)

int_h = re.compile("#include \"(.*).h\"(.*)")

sources = []
headers = []

with_python = False
with_html     = True
with_latex    = True
with_png     = False

def call(str):
  tmp = os.popen(str, "w")
  tmp.close()

def listFiles():
	global sources, headers
	os.chdir("sources")
	sources = [f for f in os.listdir(os.curdir) if (cpp.match(f,re.IGNORECASE))]
	os.chdir("../include")
	headers = [f for f in os.listdir(os.curdir) if (hpp.match(f, re.IGNORECASE))]
	os.chdir("..")

def integrity():
	for f in sources:
		try:
			o = open("sources/" + f, "r")
		except IOError:
			print "Error, file %s cannot be open" % f
		else:
			# read inclusions
			for l in o.readlines():
				m = int_h.findall(l)
				if len(m):
					include = m[0][0] + ".h"
					if include not in headers:
						print "Error, the file %s is not in the include/ path" % include
			o.close()

def make_configh():
	global with_png, with_python, with_html, with_latex
	try:
		configh = open("include/Config.h","w")
	except IOError:
		return False
	else:
  		configh.write("// Fichier genere automatiquement par Buildfile.\n// Modelib / Programmes lineaires pour C++ et Python\n")
  		configh.write("#ifndef __CONFIG_H\n#define __CONFIG_H\n\n")
  		configh.write("#define __no_export\n")
		if not (os.name == "posix" or os.name == "mac" or os.name == "riscos"):
			configh.write("#define WINDOWS_VERSION\n")
  		if with_png:
			configh.write("#define EXPORT_SPARSE_MATRIX_PNG\n")
  		if with_python:
			configh.write("#define COMPILE_FOR_PYTHON\n")
  		if with_latex:
			configh.write("#define USE_LATEX_FORMAT\n")
		if with_html:
			configh.write("#define USE_HTML_FORMAT\n")
  		configh.write("\n\n#endif\n\n")
  		configh.close()
	return True

def options():
	global with_png, with_python, with_html, with_latex
	print "+----------------------------+"
	print "| Modelib options            |"
	print "+----------------------------+"
	t = raw_input("Do you want the python support (experimental)  [yes/default=no] ? ")
	if len(t) and (t[0] == 'y' or t[0] == 'Y'):
		with_python = True
	else:
		with_python = False
	t = raw_input("Do you want the sparse matrix visualisation export (needs libpng >= 1.2)  [yes/default=no] ? ")
	if len(t) and (t[0] == 'y' or t[0] == 'Y'):
		with_png = True
	else:
		with_png = False
	t = raw_input("Do you want HTML report generation  [default=yes/no] ? ")
	if len(t) and (t[0] == 'n' or t[0] == 'N'):
		with_html = False
	else:
		with_html = True
	t = raw_input("Do you want LaTeX report generation  [default=yes/no] ? ")
	if len(t) and (t[0] == 'n' or t[0] == 'N'):
		with_latex = False
	else:
		with_latex = True
	#
	# build the Config.h file
	#
	if not make_configh():
		print "Error, Config.h must be created ! (critical error)"
		os.exit(1)

def call_comp(file, parameters):
	str = "g++ -c %s ./sources/%s -Iinclude" % (parameters, file)
	os.system(str)


def compile():
	global sources
	print "Start compiling..."
	# fix parameters
	parameters = "-O2"

	for f in sources:
		if f == "Png.cpp":
			if with_png:
				call_comp(f, parameters)
		elif comp_py.match(f, re.I) or f == "Python.cpp":
			if with_python:
				call_comp(f, parameters)
		elif f == "htmlFile.cpp":
			if with_html:
				call_comp(f, parameters)
		elif f == "latexFile.cpp":
			if with_latex:
				call_comp(f, parameters)
		else:
			call_comp(f, parameters)

	# link
	print "Start linking..."
	obj = [f for f in os.listdir(os.curdir) if (comp_o.match(f, re.IGNORECASE))]
	link_str = "g++ "
	for f in obj:
		link_str += ("%s " % f)
	link_str += " -o bin/model.a"
	call(link_str)
	call("rm *.o")
	print "Cleaning..."


if __name__ == "__main__":
	print "-> install"
	listFiles()
	print "Checking integrity of files and include system..."
	integrity()
	options()
	compile()
	print "install <- (end)"
