/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//:~ sof
#include "Config.h"
#include "FichierLP.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <list>
#include <string>
#include <algorithm>
#include <cctype>
#include <map>
#include <utility>
#include <vector>

namespace Modelib {

using namespace std;


/** Surcharge de flux d'un string
	Cette fonction a ete cree car on avait un probleme lors du passage
	des fichiers LP ecrits sous linux <-> windows.
	
	@param o		Le flux de sortie
	@param str		Le string a afficher dans le flux
	@return ostream L'actualisation du flux
*/
ostream& operator<<(ostream &o, const string& str)
{
	for (string::const_iterator iter=str.begin();iter!=str.end();++iter)
	{
		// on ignore les caracteres non affichables !
		if (*iter < 32)
			continue;
		o << *iter;
	}
	return o;
}

/**	Surchage du flux de sortie
	@param	o           flux de sortie
	@param	plExprNode  ce que le flux va afficher
    @return ostream     on retourne classiquement le flux "le flux ne peut pas etre stopp"
*/
ostream& operator<<(ostream &o, const plExprNode& pl)
{
  o << (pl.signe == MINUS ? '-' : '+') << pl.num << "*" << pl.var;
  return o;
}

/**	Surchage du flux de sortie
		@param	o           flux de sortie
		@param	plGlobalNode  ce que le flux va afficher
    @return ostream     on retourne classiquement le flux "le flux ne peut pas etre stopp"
*/
ostream& operator<<(ostream &o, const plGlobalNode& pl)
{
  if (pl.nom != "")
    o << pl.nom << ": ";
  for (list<plExprNode *>::const_iterator iter = pl.expr.begin();iter!=pl.expr.end(); ++iter)
  {
    if (iter == pl.expr.begin())
       o << ((*iter)->signe == MINUS ? "-" : " ") << (*iter)->num << '*' << (*iter)->var << ' ';
    else
       o << **iter;
  }
  o << "  " << comp[pl.cont.comp] << " " << pl.cont.bound;
  return o;
}


/**	Surchage du flux de sortie
		@param	o           flux de sortie
		@param	plConstraintNode  ce que le flux va afficher
    @return ostream     on retourne classiquement le flux "le flux ne peut pas etre stopp"
*/
ostream& operator<<(ostream& o, const plConstraintNode& pl)
{
  o << pl.minBound << " ";
  o << comp[pl.comp1] << " ";
  if (pl.num != "1")
     o << pl.num << "*";
  o << pl.var;

  if (pl.comp2 != 99)
     o << comp[pl.comp2] << " " << pl.maxBound;

  return o;
}

/**	Surchage du flux de sortie, celui l permet de ressortir
    tout ce qui est stock sous forme de fichier LP

    "" J'aime pas friend mais bon, faut bien le mettre l:

     bjarne a dit :

     > Does "friend" violate encapsulation ?

     No. It does not. "Friend" is an explicit mechanism for granting access, just like membership.
     You cannot (in a standard conforming program) grant yourself access to a
     class without modifying its source.
    ""

	@param	o           flux de sortie
	@param	FichierProbleme  ce que le flux va afficher
    @return ostream     on retourne classiquement le flux "le flux ne peut pas etre stopp"
*/
ostream& operator<<(ostream &o, const FichierProbleme& pblm)
{
    o << "\\Sortie du fichier de probleme: " << pblm.nom << endl;

    // Affichage de la fonction objectif
    if (pblm.isMax == true)
       o << "Maximize" << endl;
    else
       o << "Minimize" << endl;

    for (list<plExprNode *>::const_iterator iter  = pblm.exprObj.begin();
                                            iter != pblm.exprObj.end()  ;
                                          ++iter)
      if (iter == pblm.exprObj.begin() && (*iter)->var.length())
         o << "\t" << ((*iter)->signe == MINUS ? "-" : " ") << (*iter)->num << "*" << (*iter)->var << ' ';
      else
         o << **iter;

    // Affichages des contraintes (fonctions)
    o << endl;
    o << "Subject to" << endl;


    for (list<plGlobalNode *>::const_iterator iter  = pblm.exprConstraint.begin();
                                            iter != pblm.exprConstraint.end()  ;
                                          ++iter)
         o << "\t" << **iter << endl;

    if (!pblm.exprBounds.empty())
    {
      o << "Bounds" << endl;
  
  
      for (list<plConstraintNode *>::const_iterator iter  = pblm.exprBounds.begin();
                                              iter != pblm.exprBounds.end()  ;
                                            ++iter)
           o << "\t" << **iter << endl;
    }
    if (!pblm.exprGeneral.empty())
    {
      o << "Generals" << endl;
      for (list<string>::const_iterator iter =pblm.exprGeneral.begin();
                                        iter!=pblm.exprGeneral.end()  ;
                                      ++iter)
           o << "\t" << *iter << endl;
    }

    if (!pblm.exprBinary.empty())
    {
      o << "Binaries" << endl;
      for (list<string>::const_iterator iter =pblm.exprBinary.begin();
                                        iter!=pblm.exprBinary.end()  ;
                                      ++iter)
           o << "\t" << *iter << endl;
    }

    o << "End";
    return o;
}



/**	
	Constructeur par dfaut !
	On y initilise notre multi map.
	
	note: utiliser map<>.insert(map<>::value_type(....)) serait mieux
	car il y a de reels problemes de copie lors de l'insertion dans un map
*/
FichierLP::FichierLP()
{

  mmap.insert(pair<string,unsigned>("maximize",MAXIMIZE));
  mmap.insert(pair<string,unsigned>("max",MAXIMIZE));

  mmap.insert(pair<string,unsigned>("minimize",MINIMIZE));
  mmap.insert(pair<string,unsigned>("min",MINIMIZE));

  mmap.insert(pair<string,unsigned>("subject to",SUBJECT));
  mmap.insert(pair<string,unsigned>("subject",SUBJECT));
  mmap.insert(pair<string,unsigned>("such that",SUBJECT));
  mmap.insert(pair<string,unsigned>("st",SUBJECT));
  mmap.insert(pair<string,unsigned>("st.",SUBJECT));
  mmap.insert(pair<string,unsigned>("s.t.",SUBJECT));

  mmap.insert(pair<string,unsigned>("bounds",BOUNDS));
  mmap.insert(pair<string,unsigned>("bound",BOUNDS));

  mmap.insert(pair<string,unsigned>("end",END));

  mmap.insert(pair<string,unsigned>("generals",GENERALS));
  mmap.insert(pair<string,unsigned>("general",GENERALS));
  mmap.insert(pair<string,unsigned>("gen",GENERALS));
  mmap.insert(pair<string,unsigned>("integers",GENERALS));
  mmap.insert(pair<string,unsigned>("integer",GENERALS));

  mmap.insert(pair<string,unsigned>("binaries",BINARIES));
  mmap.insert(pair<string,unsigned>("binary",BINARIES));
  mmap.insert(pair<string,unsigned>("bin",BINARIES));

  mmap.insert(pair<string,unsigned>("free",FREE));

  // remplir le Comp2Int
  for (unsigned i=0;i<6;i++)
      Comp2Int[comp[i]] = i;
}

/**	Est-ce que le string s est un commentaire ?
		@param	s           Le string sera modifi s"il contient un commentaire, celui-ci sera effac
    @return bool = vrai si c'est une ligne complte de commentaire
*/
bool FichierLP::IsComment(string& s)
{
  // on return false si c pas un comentaire complet
  // et on efface tout ce qu'il y a apres
  string::iterator iter = s.begin();
  string           tmp;
  // si c'est une ligne complete
  if (*iter == '\\') {
  	 commentaires.push_back( pair<long,string>( (long)exprConstraint.size() 
     + ( exprObj.size()==0?0:1) ,s) );
     return true;
  }
  // sinon... on gicle tout ce qu'il y a apres le char
  for (;iter!=s.end();++iter)
  {
    if (*iter == '\\')
       break;
    tmp += *iter;
  }
  s.erase();
  s = tmp;
  return false;
}

/**	Eliminer les espaces des bords
		@param	s        le string a modifier
    @return string = le string sans espaces sur les cots
*/
string FichierLP::trim(const string& s) const
{
  if(s.empty())
    return s;
  int b = s.find_first_not_of(" ");
  int e = s.find_last_not_of(" ");
  if(b == -1)
    return "";
  return string(s, b, e - b + 1);
}

/**	Eliminer tous les espaces, on concatenne tous les mots
		@param	s        le string a modifier
    @return string = le string sans aucuns espaces
*/
string FichierLP::AvoidSpaces(const string& s) const
{
  string t;
  string::const_iterator iter = s.begin();
  for(;iter!=s.end();++iter)
    if (*iter != ' ')
      t += *iter;
  return t;
}

/**	Teste si le string est un nombre valide pour le programme. Les nombres valides sont
    des nombres entiers, fractionnaires ou flottants
	@param	s        le nombre a tester
    @return bool = vrai si c'est un nombre valide
*/
bool FichierLP::IsNumber(const string& s) const
{
  const string nums="0123456789./";
  string::const_iterator iter;
  bool pointWOSlash = false;
  for (iter=s.begin();iter!=s.end();++iter)
  {
    if (pointWOSlash && *iter=='.')
      return false;
    if (nums.find(*iter) != string::npos)
    {
      if (*iter == '.')  pointWOSlash=true;
      if (*iter == '/' ) pointWOSlash=false;
    }
    else
      return false;
  }
  return true;
/* tests effectus l dessus
  cout << endl;
  cout << "2.000 = " << IsNumber("2.000") << endl;
  cout << "2.000. = " << IsNumber("2.000.") << endl;
  cout << "2.000/1 = " << IsNumber("2.000/1") << endl;
  cout << "2.000/0.00 = " << IsNumber("2.000/0.00") << endl;
  cout << "2.000/0.00. = " << IsNumber("2.000/0.00.") << endl;
  cout << endl;
*/
}

/**	Spare une expression a-b-c+d en deux listes {a,d} et {b,c}
		@param	str          le string  dcomposer
		@param  tokens_plus  stockages des monomes commenants par un '+'
		@param  tokens_minus stockages des monomes commenants par un '-'
*/
void FichierLP::Tokenize(const string& str,list<string>& tokens_plus
                                          ,list<string>& tokens_minus)
{
    enum Token_Value {PLUS='+', MINUS='-'};
    Token_Value cur_token = PLUS;
    string tmp;
    string::size_type i=0;
    const string::size_type len = str.length();

    do
    {
      while(str[i]==' ') ++i;
      if (str[i] == PLUS || str[i] == MINUS)
      {
        if(tmp.length())
        {
        	// on le balance dans le container de tokens
            switch(cur_token)
            {
              case PLUS :  tokens_plus.push_back(tmp); break;
              case MINUS:  tokens_minus.push_back(tmp); break;
              default   :                              break;
            }
            // alors on flush tmp
            tmp.clear();
            // actualisation du token
            cur_token = (str[i]=='+'?PLUS:MINUS);
        }
        else
          cur_token = (str[i]=='+'?PLUS:MINUS);
      }
      else
        tmp += str[i];
      ++i;
    }
    while (i < len);
    
    // il reste necessairement la derniere partie 
    switch(cur_token)
    {
      case PLUS :  tokens_plus.push_back(tmp); break;
      case MINUS: tokens_minus.push_back(tmp); break;
      default   :                              break;
    }
}

/**	Extraire le nombre en facteur et la variable d'un monme
		@param	str          le string  dcomposer
		@param  num          contiendra le nombre mis en facteur
		@param  var          contiendra la variable
*/
void FichierLP::GetNumVar(const string& str, string& num, string& var)
{
  string::const_iterator iter=str.begin();
  unsigned i=0;
  if (!isdigit(*iter))
  {
    num = "1";
    var = str;
  }
  else
  {
    for (;iter!=str.end();++iter,++i)
    {
      if (!IsNumber(str.substr(0,i)))
      {
        num=str.substr(0,i-1);
        var=str.substr(i-1,str.length());
        break;
      }
    }
  }
}




/**	Parser l'expression de la fonction objectif !
		@param	s    le string  anaylser
*/
void FichierLP::ParseObjective(const string& s)
{
  // on va tokeniser par +/-
  list<string> token_plus;
  list<string> token_minus;
  list<string>::iterator iter;

  if (s.length() <= 1)
     return;

  Tokenize (s,token_plus,token_minus);

  for(iter=token_plus.begin();iter!=token_plus.end();++iter)
  {
    string var,num;
    GetNumVar(*iter,var,num);

    //cout << "PLUS: "<< var << '(' << trim(var) << ')' << '(' << var.size() << ')' << endl;

    auto_ptr<plExprNode> exprNode (new plExprNode (num,var,PLUS)) ;
    exprObj.push_back(exprNode.release());
  }

  for(iter=token_minus.begin();iter!=token_minus.end();++iter)
  {
    string var,num;
    GetNumVar(*iter,var,num);
    //cout << "PLUS: "<< var << '(' << trim(var) << ')' << '(' << var.size() << ')' << endl;

    auto_ptr<plExprNode> exprNode (new plExprNode (num,var,MINUS)) ;
    exprObj.push_back(exprNode.release());
  }

  // ca marche !
  //cout << "Et l ?" << endl;
  // afficher les donnes
  //for (list<plExprNode *>::iterator i=exprObj.begin(); i!=exprObj.end(); ++i)
  //    cout << **i << endl;


}


/**	Parser une expression d'une contrainte
		@param	s    le string  anaylser
		@param  nom  le nom de la contrainte s'il existe
*/
void FichierLP::ParseSubjects (const string& s, const string& nom)
{
  // contraintes SubJects...
  // trouver le dlimitage :]
  string::size_type place_comp=string::npos;
  unsigned i=0;
  

    if (s.length() <= 1)
     return;


  for (;i<6;i++)
  {
    if ((place_comp = s.find(comp[i])) != string::npos)
       break;
  }

  if (place_comp == string::npos)
  {
    // bah ca merde !
    return;
  }

  string compareTo;
  string second;
  compareTo = s.substr(place_comp+comp[i].length(),s.length());
  second    = s.substr(0,place_comp);

  //cout << i << " chaine=" << second << "| compare a=" << compareTo << " par " << comp[i] << endl;

  list<string> token_plus;
  list<string> token_minus;
  list<string>::iterator iter;
  Tokenize (second,token_plus,token_minus);

  // ce qu'on va pusher !
  //auto_ptr<plGlobalNode> tmpNode;
  list<plExprNode *>     tmpObj;

  for(iter=token_plus.begin();iter!=token_plus.end();++iter)
  {
    string var,num;
    GetNumVar(*iter,var,num);
    auto_ptr<plExprNode> exprNode (new plExprNode (num,var,PLUS)) ;
    tmpObj.push_back(exprNode.release());
  }

  for(iter=token_minus.begin();iter!=token_minus.end();++iter)
  {
    string var,num;
    GetNumVar(*iter,var,num);
    auto_ptr<plExprNode> exprNode (new plExprNode (num,var,MINUS)) ;
    tmpObj.push_back(exprNode.release());
  }

  // ca, ca nous ressort bien ce qu'on veut !
  // ie la liste des tokens
  //cout << "Ce qu'on devrait avoir..." << endl;
  //for (list<plExprNode *>::iterator i=tmpObj.begin(); i!=tmpObj.end(); ++i)
  //    cout << **i << endl;

  // Creation de notre ptr qui sera push
  plSimpleConstraint tmpConst(compareTo,i);
  auto_ptr<plGlobalNode> tmpNode (new plGlobalNode (nom,tmpObj,tmpConst));

  // gnourf ?
  //cout << "Ce qu'on a :|" << endl;

  // afficher les donnes
  //cout << "Choppage en manuel des 'values'" << endl;
  //list<plExprNode *>::const_iterator iterNode=tmpNode.get()->expr.begin();

  // boucle avec l'itrateur
  //cout << "et avec l'itrateur.. et l, c le drame" << endl;
  //for (iterNode=tmpNode.get()->expr.begin();iterNode!=tmpNode.get()->expr.end(); ++iterNode)
  //    cout << **iterNode << endl;

  //cout << *(tmpNode.get()) << endl;

  exprConstraint.push_back(tmpNode.release());
}


/**	Obtenir le nombre de comparateur qu'il existe dans l'expression
		@param	_str  le string  anaylser
		@return unsigned = le nombre de comparateurs
*/
unsigned GetNbComparator(const string& _str)
{
  unsigned nbComp=0;
  string::size_type pos;
  string sub=_str;

  for (unsigned j=0;j<6;)
  {
    if ((pos=sub.find(comp[j])) != string::npos)
    {
      nbComp++;
      sub = sub.substr(pos+comp[j].length(),sub.length());
    }
    else
      j++;
  }
  return nbComp;
}


/**	Parser l'encadrement des contraintes.
    Elle sont de plusieurs types:
               b <= bsup
       binf <= b <= bsup
       binf <= b
		@param	s  le string  anaylser
*/
void FichierLP::ParseBounds(const string& s)
{
  // parser :
  // -Inf <= a <= bsup
  //         b <= bsup
  // binf <= c
  string str;

  if (s.length() <= 1)
     return;


  string::const_iterator iter,it2=s.begin(),next;
  // compter le nombre de de signes
  unsigned nbComp = GetNbComparator(s);
  // 2*nbComp+1 champs !

  // on ajoute des espaces pour utiliser les stringstream
  // iter: cour
  // it2 : prec
  next = s.begin(); ++next;
  for (iter=s.begin();iter!=s.end();++iter, ++next)
  {
      switch(*iter)
      {
        case '<': // Si <
        case '>': // ou >
             str.push_back(' ');   // on ajoute l'espace
             str.push_back(*iter); // et < ou >
                
                // ajouter un espace s'il n'y a pas de signe apres
                if (*next != '=')
                   str.push_back(' ');
             break;

        case '=':
             if (*it2 != '<' && *it2 != '>')
               str.push_back(' ');
             str.push_back(*iter);
             str.push_back(' ');
             break;

        default:
             str.push_back(*iter);
             break;
      }
      it2 = iter;
  }

  string *t = 0;
  if (nbComp > 0)
  {
    istringstream sstring(str);
    t = new string[(2*nbComp+1)];
  
    for (unsigned i=0;i<(nbComp<<1) + 1;i++)
    {
        sstring >> t[i];
    }
  }


  // on va remplir le exprBounds
  // exprBounds
  switch (nbComp)
  {
    case 2:
      {
          // 2 comparateurs => min <= var <= max
          //plConstraintNode(const std::string& _var, const std::string& _num,
          //                   const std::string& _b1 , const std::string& _b2 ,
          //                   const unsigned     _c1 , const unsigned     _c2)
         auto_ptr<plConstraintNode> cNode (new plConstraintNode (t[2],"1",t[0],t[4],Comp2Int[t[1]],Comp2Int[t[3]]));
         exprBounds.push_back(cNode.release());
      }
    break;

    case 1:
      {
         // si t[0] est un nombre alors
         //   forme 0 <= var      _num _var _minbound _comp1
         // sinon
         //   autre forme         _num _var _comp1 _maxbound
         //cout << "t0 = " << t[0] << endl;
         if (IsNumber(t[0]))
         {
           auto_ptr<plConstraintNode> cNode (new plConstraintNode ("1",t[2],t[0],Comp2Int[t[1]]));
           exprBounds.push_back(cNode.release());
         }
         else
         {
           auto_ptr<plConstraintNode> cNode (new plConstraintNode ("1",t[0],t[2],(Comp2Int[t[1]]+2)%4));
           exprBounds.push_back(cNode.release());
         }
      }
    break;
    
    case 0:
     {
        // Le free est ici
        // x1 Free
        string tmp;
        istringstream sstring(str);
        sstring >> tmp;
        auto_ptr<plConstraintNode> cNode (new plConstraintNode (tmp,"1","-Inf","Inf",LOWER_THAN,LOWER_THAN));
        exprBounds.push_back(cNode.release());
     }
    break;

    default:
        // on levera une exception ici pke ca ne peut pas exister !
        // enfin, je pense sincremetn que CPlex ne gere pas :
        // MIN <= x1 <= x2 <= x3 <= ... <= MAX
    break;
  }
  if (t)
    delete [] t;
}

/**	Parser le champ Generals
		@param	s  le string  anaylser
*/
void FichierLP::ParseGenerals(const string& s)
{
     // on a une ligne x2,x3,x4...
     // ou  x1
     //     x2
     //     x3

     // 1/ on va donc se normaliser l'expression en x4 x2 etc. sur une ligne
     // 2/ utiliser un stringstream et pusher le tout dans la liste


    if (s.length() <= 1)
       return;
    
    
    // 1/ Il suffit de remplacer toutes les virgules par des espaces :]
    string str;
    string::const_iterator iter=s.begin();
    for (;iter!=s.end(); ++iter)
    {
      if (*iter == ',')
         str.push_back(' ');
      else
         str.push_back(*iter);
    }
    
    // 2/ istringstream est sympatique :]
    istringstream istring(str);
    string tmp;
    while (istring >> tmp) {
         exprGeneral.push_back(tmp);
         tmp.erase();
    }
}


/**	Parser le champ Binaries
		@param	s  le string  anaylser
*/
void FichierLP::ParseBinaries(const string& s)
{
     // on a une ligne x2,x3,x4...
     // ou  x1
     //     x2
     //     x3

  if (s.length() <= 1)
     return;


     // 1/ on va donc se normaliser l'expression en x4 x2 etc. sur une ligne
     // 2/ utiliser un stringstream et pusher le tout dans la liste


     // 1/ Il suffit de remplacer toutes les virgules par des espaces :]
     string str;
     string::const_iterator iter=s.begin();
     for (;iter!=s.end(); ++iter)
     {
        if (*iter == ',')
           str.push_back(' ');
        else
           str.push_back(*iter);
     }
     
     // 2/ istringstream est sympatique :]
     istringstream istring(str);
     string tmp;
     while (istring >> tmp) {
           exprBinary.push_back(tmp);
           tmp.erase();
     }
}



/**	Ouvrir le fichier LP
		@param	nom  le nom du fichier  ouvrir
		       @return bool = vrai si kle fichier a t trait
*/
bool FichierLP::Open(const string& nomFile)
{
  ifstream ifile(nomFile.c_str());

  string   s,tempInlineString;
  multimap<string,unsigned>::iterator k;
  unsigned oldK=42;
  bool continu=false,f=false;
  bool objectiveInline = false;


  if (!ifile.is_open())
     return false;

  //while (getline(ifile,s))
  //      cout << s << endl;
  //ifile.clear();
  // on revient au dbut apres avoir affich le source lp
  //if (!ifile.seekg(0,ifstream::beg))
  //   return false;

  while (getline(ifile,s,'\n'))
  {
    if (!s.empty() && IsComment(s) == false)
    {
    	string::size_type position = s.rfind('\r');
    	if (position != string::npos)
    		s.replace(position,1," ");

      //// on fait un tolower pour pas se faire emmerder
      // par la casse
      if (objectiveInline)
      {
        objectiveInline = false;
        // on traite tempInelinestring comme un ParseObjective
        ParseObjective(tempInlineString);
      }

      //string c=s;
      string c;
      for (string::const_iterator iter = s.begin(); iter!=s.end();++iter)
      {
        if (*iter == ' ')
           break;
        c.push_back(*iter);
      }

      transform (c.begin(),c.end(),c.begin(),::tolower);
      // On peut maintenant travailler sur le "string s"

      k = mmap.find(trim(c)); // Recherche de la value !s
      if (k == mmap.end())    // si value non trouve dans la table
         continu = true;
      else                    // c'est que ce n'est pas un nouveau champ
          continu = false,
          oldK = 42;

      if (!continu && k->second == END)
      {

        // affichage des tous les commentaires
        /*for (list<string>::iterator iC =commentaires.begin();
                                    iC!=commentaires.end();
                                    ++iC)
            cout << "Commentaires: " << *iC << endl;
        */

         return true;
      }

      if (oldK==42 && objectiveInline==false && ((k->second == MAXIMIZE && oldK != MAXIMIZE) || (k->second == MINIMIZE && oldK != MINIMIZE)))
      {
        objectiveInline = true;
        tempInlineString = s.substr(8,s.length());
      }

      switch(oldK)
      {
         //
         // on doit grer le multiligne
         // ce n'est pas encore fait pour les contraintes
         // ca devrait y etre bientot ...
         // quand tous les champs seront grs
         //
         //                                         13/12/2004
         case MINIMIZE:
         case MAXIMIZE:
      	 {
      	   if (f==false){
        	   if (oldK==MINIMIZE) isMax=false, f=true;
             else                isMax=true,  f=true;
           }
           // string | list<lpExprNode *> | plSimpleConstraint
           size_t pos;
           string _nom="(null)";

           s = AvoidSpaces(s);
           // on va avancer jusqu'aux ':'
           if ((pos = s.find(':',0)) != string::npos)
           {
             _nom = s.substr(0,pos);
             nom = _nom;
             s.erase(0,pos+1);
           }
           ParseObjective(s);
        	 break;
         }
         case SUBJECT:
         {
           size_t pos;
           string nom;

           s = AvoidSpaces(s);
           if ((pos = s.find(':',0)) != string::npos) {
             nom = s.substr(0,pos);
             s.erase(0,pos+1);
           }
           ParseSubjects(s,nom);
        	 break;
         }
      	 case BOUNDS:
             ParseBounds(trim(s));
        	 break;
      	 case GENERALS:
             ParseGenerals(trim(s));
        	 break;
      	 case BINARIES:
             ParseBinaries(trim(s));
        	 break;
      	 case END:
      	   break;
      	 default:
           break;
      }

      if (continu == false)
        oldK = k->second;
    }
  }

  return true;
}

}

//:~ eof
