/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "FichierProbleme.h"
#include "Model.h"

#include <ctime>

namespace Modelib {


using namespace std;

/**Oprateur de surcharge de flux permettant de charger un modle
    partir un fichier problme
  @param m  le modle dans lequel le FichierProbleme doit tre charg
  @return vrai si il n'y a pas eu de probleme
 */
bool FichierProbleme::operator>> (Model & m)
{
  map<string,NumVar> table;
  //table permettant de faire correspondre un nom de variable "texte" 
  //du fichier probleme  une variable du modele
  map<string,NumVar>::iterator cour;
  //itrateur sur cette table

  list<plExprNode *>::iterator obj =  exprObj.begin();
  //itrateur permettant de parcourir la fonction objectif "texte"
  list<plGlobalNode *>::iterator cons = exprConstraint.begin();
  //itrateur permettant de parcourir les contraintes "texte"
  list<string>::iterator gen = exprGeneral.begin();
  //itrateur permettant de parcourir les variables entieres "texte"
  list<string>::iterator bin = exprBinary.begin();
  //itrateur permettant de parcourir les variables boolennes "texte"
  list<plConstraintNode *>::iterator bou =  exprBounds.begin();
  //itrateur permettant de parcourir les bornes "texte"

  cerr << "FichierProbleme : Loading" << endl;
  cerr << "Generals";
  //Pour chaque nom de variable dans Generals
  for( ; gen != exprGeneral.end(); ++gen )
  {
    cour = table.find( *gen );
    //on regarde si la variable existe
    if( cour != table.end() )
  	{
       cour->second.SetType(MuteVar::INT);
       //on change son type si elle existe dj
    }
  	else
  	{
        NumVar x(m,0,Infinity,MuteVar::INT,*gen);
        table[*gen] = x;
        //sinon on la cre et on la rpertorie
  	}
   }
  cerr << "[OK]" << endl;

  cerr << "Booleans";
  //Pour chaque nom de variable dans Booleans
  for( ; bin != exprBinary.end(); ++bin )
  {
  	cour = table.find( *bin );
    //on regarde si la variable existe
  	if( cour != table.end() )
  	{
       cour->second.SetType(MuteVar::BOOL);
       //on change son type si elle existe dj
       cour->second.SetLowerBound(0);
       cour->second.SetUpperBound(1);
       //On change ses bornes
   }
  	else
  	{
        NumVar x(m,0,1,MuteVar::BOOL,*bin);
        table[*bin] = x;
        //sinon on la cre et on la rpertorie
  	}
  }
  cerr << "[OK]" << endl;
  
  cerr << "Objectif";
  Expr tmp; 
  //Expression temporaire qui va servir  construire la contrainte
  //Pour toutes les groupes coefficient * variable de la fonction objectif
  for( ; obj != exprObj.end(); ++obj )
  {
    cour = table.find((*obj)->var);
    //on regarde si la variable existe
    float coef = atof( (*obj)->num.data() );
  	//on convertit le coefficient "texte" en float
    coef = ((*obj)->signe?coef:-coef);
    //on change son signe au cas ou il y ait "-" devant
  	if( cour != table.end() )
  	{
  		tmp.ASS(coef,cour->second);
  		//On ajoute le tout  l'expression temporaire si elle existe
  	}
  	else
  	{
        NumVar x(m,0,Infinity,MuteVar::FLOAT,(*obj)->var);
        table[(*obj)->var] = x;
        //sinon on la cre et on la rpertorie
        tmp.ASS(coef,x);
  		//et on ajoute le tout  l'expression temporaire si elle existe
  	}
  }
  
  //si la fonction est une maximisation
  if(isMax == true)
     m.Maximize( tmp, nom );
     //on dfinit cette fonction pour une maximisation
  else
     m.Minimize( tmp, nom );
     //on dfinit cette fonction pour une minimisation

  cerr << "[OK]" << endl;

  cerr << "Constraints";

  //Pour toutes les contraintes
  for( ; cons != exprConstraint.end(); ++cons )
  {
    tmp = Expr();
    //On rintialise l'expression temporaire
    //Pour toutes les groupes coefficient * variable de la contrainte
    for( obj = (*cons)->expr.begin(); obj != (*cons)->expr.end(); ++obj )
    {
        cour = table.find((*obj)->var);
        //on regarde si la variable existe
    	float coef = atof( (*obj)->num.data() );
  	    //on convertit le coefficient "texte" en float
    	coef = ((*obj)->signe?coef:-coef);
        //on change son signe au cas ou il y ait "-" devant
        if( cour != table.end()  )
    	{
    		  tmp.ASS(coef,(cour->second));
  		      //On ajoute le tout  l'expression temporaire si elle existe
    	}
    	else
    	{
            NumVar x(m,0,Infinity,MuteVar::FLOAT,(*obj)->var);
            table[(*obj)->var] = x;
            //sinon on la cre et on la rpertorie
            tmp.ASS(coef,x);
  		    //et on ajoute le tout  l'expression temporaire si elle existe
    	}
    }

    float coef = atof( (*cons)->cont.bound.data() );
    //on extrait le second membre de contrainte "texte"

    //suivant son type on l'ajoute au modele avec le bon operateur
    switch((*cons)->cont.comp)
    {
        case 0:
            m.Add( tmp <= coef, (*cons)->nom );
        break;
        case 1:
            m.Add( tmp < coef, (*cons)->nom);
        break;
        case 2:
            m.Add( tmp >= coef,(*cons)->nom );
        break;
        case 3:
            m.Add( tmp > coef, (*cons)->nom);
        break;
        case 4:
            m.Add( tmp != coef,(*cons)->nom );
        break;
        case 5:
            m.Add( tmp == coef, (*cons)->nom);
        break;
        default:
            cerr << endl <<"Warning in FichierProbleme::>> : undefined constraint type" << endl;
        break;
    }
  }
  cerr << "[OK]" << endl;

  cerr << "Bounds";
  //Pour chaque borne
  for( ; bou != exprBounds.end(); ++bou )
  {
  	cour = table.find( (*bou)->var );
  	//on regarde si la variable existe
  	if( cour == table.end() )
  	{
        NumVar x(m,0,Infinity,MuteVar::FLOAT,(*bou)->var);
        table[(*bou)->var] = x;
        //sinon on la cre et on la rpertorie
        tmp = x;
        //on la considere comme une expression
  	}
  	else
  	   tmp = cour->second;
  	   //on la considere comme une expression

     float lB=0,uB=0;
    
     //si la borne infrieur est l'infini
     if( (*bou)->minBound == "-inf" || (*bou)->minBound == "-Inf")
        lB = -Infinity;
        //on la definit avec l'infini du modele
     else
        lB = atof( (*bou)->minBound.data() );
        //si on extrait sa valeur de sa reprsentation texte
    
     //si la borne suprieur est l'infini
     if( (*bou)->maxBound == "+inf" || (*bou)->maxBound == "Inf")
        uB = Infinity;
        //on la definit avec l'infini du modele
     else
        uB = atof( (*bou)->maxBound.data() );
        //si on extrait sa valeur de sa reprsentation texte
    
     ConstraintBuilder constraint;
     //on declare un constructeur de contrainte
    
     //on regarde le type de borne  gauche et on cre la bornes en consquence
     switch( (*bou)->comp1 )
     {
          case 0:
              constraint = lB <=  tmp;
          break;
          case 1:
              constraint = lB <  tmp;
          break;
          case 2:
              constraint =  lB >= tmp ;
          break;
          case 3:
              constraint = lB > tmp;
          break;
          case 4:
              constraint = lB !=  tmp;
          break;
          case 5:
              constraint = lB ==  tmp;
          break;
          default:
              cerr << endl << "Warning in FichierProbleme::>> : undefined bounds type" << endl;
          break;
     }
    
     //on regarde le type de borne  droite et on cre la bornes en consquence
     switch( (*bou)->comp2 )
     {
          case 0:
              constraint <= uB ;
          break;
          case 1:
              constraint < uB ;
          break;
          case 2:
              constraint >= uB ;
          break;
          case 3:
              constraint > uB ;
          break;
           case 4:
              constraint != uB ;
          break;
          case 5:
             constraint == uB ;
          break;
          case 99:
    
          break;
          default:
              cerr << endl << "Warning in FichierProbleme::>> : undefined bounds type" << endl;
          break;
    }
    
     m.Add( constraint );
     //On l'ajoute au modele
  }
  cerr << "[OK]" << endl;
  cerr << "Comments";
  
  list<pair<long,string> >::iterator comment =  commentaires.begin();
  for( ; comment != commentaires.end(); ++comment )
  {

  	if( comment->first == 0 )
    {
      string name = m.GetProblemName();
      if( name.size() == 0 )
      {
        comment->second.erase( comment->second.begin() );
        m.SetProblemName( comment->second );
      }
      else
      {
        m.SetProblemName( name + "\n" + comment->second );
      }
    }
    else
    {
  	  string tmp = m.GetComment( comment->first );
      if( tmp.size() == 0 )
    	{
    		m.SetComment( comment->first, comment->second );
    	}
    	else
    	{
    		m.SetComment( comment->first, tmp + "\n" + comment->second );
    	}
    }
  }

  cerr << "[OK]" << endl;
  cerr << "Loading Complete";

  return true;
}

}
