/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Fraction.h"
#include <map>
#include <iostream>

namespace Modelib {

using namespace std;

/**
	Initialisation de note convertisseur de fraction
	@param n La precision de l'approximation
*/
FractionConvert::FractionConvert(unsigned _n)
{
 map<float,Fraction> tmp;
 map<float,Fraction>::iterator trouve;
 for(unsigned i = 0; i <_n; ++i)
 {
   for(unsigned j = 0; j <i; ++j)
   {
    trouve = tmp.find(float(j)/float(i));
    if( trouve == tmp.end() )
      tmp[float(j)/float(i)] = Fraction(j,i);
    else if( trouve->second.num > i )
      trouve->second = Fraction(j,i);
   }
 }
 nfrac = tmp.size();
 frac = new Fraction[nfrac];
 reel = new float[nfrac];

 unsigned i =0;  
 for(trouve = tmp.begin(); trouve != tmp.end(); ++trouve)
 {
   reel[i] = trouve->first;
   frac[i] = trouve->second;
   ++i;
 }
}

/**
	Destructeur
*/
FractionConvert::~FractionConvert()
{
  delete [] frac;
  delete [] reel; 
}


/**
	Approximer
	@param nb	Le nombre a approxmimer
	@param num	Le numerateur qui sera retourne
	@param den	Le denominateur qui sera retourne
	
	@return true si l'operation a ete effectuee avec succes
*/
bool FractionConvert::Approx(float nb, int & num, unsigned  & den) const
{
   unsigned a = 0,b=nfrac-1;
   if(nb==0.0)
   {
       num = 0;
       den = 1;
       return false;
   }

   while(a<b)
   {
     unsigned mil = (a+b)/2;
     if(reel[mil] < nb)
        a = mil + 1;
     else
        b = mil;
   }
   if(a < nfrac)
   {
     if( reel[a]-nb >= nb - reel[a-1] )
       a--;
     num = frac[a].num;
     den = frac[a].den;
     return true;
   }
   else
     return false;
}

}
