/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "PsFile.h"
#include <cmath>

namespace Modelib {


using namespace std;



void PSFile::begin_figure(
    double _xmin, double _xmax,
    double _ymin, double _ymax,

    double _hmin, double _hmax,
    double _vmin, double _vmax,

    int xn, int yn
  )
  {
    
    save_scales(
      _xmin, _xmax, _ymin, _ymax,
      _hmin, _hmax, _vmin, _vmax
    );
    
    char *date = asctime( localtime( NULL ) );
    
    psfile <<  "%%!PS-Adobe-2.0 EPSF-2.0" << endl;
    psfile <<  "%%%%CreationDate: "<< date << endl;
    psfile <<  "%%%%BoundingBox: "<< hmin<<' '<<vmin<<' '<<hmax<<' '<<vmax<<endl;
    psfile <<  "%%%%Pages: 1" << endl;
    psfile <<  "%%%%EndComments" << endl;
    
    psfile <<  "/$maindict 6400 dict def " << endl;
    psfile <<  "$maindict begin" << endl;
    
    define_procs();
    define_caption_procs();
    
    psfile <<  "end" << endl;
    psfile <<  "%%%%EndProlog" << endl;

    psfile <<  "%%%%Page: 1" << endl;
    psfile <<  "$maindict begin" << endl;
    psfile <<  "/savedstate save def" << endl;

    setup_page_state(xn, yn);
    setup_caption_data();

    psfile.flush();
  }
  
void PSFile::end_figure()
  {
    aux_end_page();
    psfile <<  "%%%%Trailer" << endl;
    psfile.flush();
  }

void PSFile::begin_document()
  {
    psfile <<  "%%!PS-Adobe-2.0" << endl;
    psfile <<  "%%%%Pages: (atend)" << endl;
    psfile <<  "%%%%EndComments" << endl;
    
    psfile <<  "/$maindict 6400 dict def " << endl;
    psfile <<  "$maindict begin" << endl;
    
    define_procs();
    define_caption_procs();
    
    psfile <<  "end" << endl;
    psfile <<  "%%%%EndProlog" << endl;

    psfile.flush();
  }

void PSFile::end_document(int npages)
  {
    psfile <<  "%%%%Trailer" << endl;
    psfile <<  "%%%%Pages: "<< npages << endl;
    psfile.flush();
  }

void PSFile::begin_page(
    int page,
    double _xmin, double _xmax,
    double _ymin, double _ymax,

    double _hmin, double _hmax,
    double _vmin, double _vmax,

    int xn, int yn
  )
  {
    char *date = "";
    save_scales(
      _xmin, _xmax, _ymin, _ymax,
      _hmin, _hmax, _vmin, _vmax
    );

    psfile <<  "%%%%Page: "<< page << ' ' << page << endl;
    psfile <<  "" << endl;

    psfile <<  "$maindict begin" << endl;
    psfile <<  "/savedstate save def" << endl;

    psfile <<  "%% Print date:" << endl;
    psfile <<  "gsave" << endl;
    psfile <<  "  /Courier findfont" << endl;
    psfile <<  "  10 scalefont setfont" << endl;
    psfile <<  "  80 18 moveto" << endl;
    psfile <<  "  ("<<date<<"  page "<<page<<") show"<<endl;
    psfile <<  "grestore" << endl;
    psfile <<  "" << endl;

    setup_page_state(xn, yn);
    setup_caption_data();
  }

void PSFile::end_page()
  {
    psfile <<  "showpage" << endl;
    psfile <<  "" << endl;
    aux_end_page();
  }

void PSFile::save_scales(
    double _xmin, double _xmax,
    double _ymin, double _ymax,
    double _hmin, double _hmax,
    double _vmin, double _vmax
  )
  {
    hmin = _hmin;
    hmax = _hmax;
    xmin = _xmin;
    xmax = _xmax;
    xscale = (hmax - hmin)/(xmax - xmin);

    vmin = _vmin;
    vmax = _vmax;
    ymin = _ymin;
    ymax = _ymax;
    yscale = (vmax - vmin)/(ymax - ymin);

  }

void PSFile::define_procs()
  {
    psfile <<  "%% Rectangle draw op:" << endl;
    psfile <<  "%%   /xlo/ /xhi/ /ylo/ /yhi/ recd --> " << endl;
    psfile <<  "/recd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "  newpath" << endl;
    psfile <<  "    3 index 2 index moveto" << endl;
    psfile <<  "    2 index 2 index lineto" << endl;
    psfile <<  "    2 index 1 index lineto" << endl;
    psfile <<  "    3 index 1 index lineto" << endl;
    psfile <<  "    pop pop pop pop" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Rectangle fill op:" << endl;
    psfile <<  "%%   /xlo/ /xhi/ /ylo/ /yhi/ /gray/ recf --> " << endl;
    psfile <<  "/recf" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    setgray" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    3 index 2 index moveto" << endl;
    psfile <<  "    2 index 2 index lineto" << endl;
    psfile <<  "    2 index 1 index lineto" << endl;
    psfile <<  "    3 index 1 index lineto" << endl;
    psfile <<  "    pop pop pop pop" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    fill" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Rectangle fill and stroke op:" << endl;
    psfile <<  "%%   /xlo/ /xhi/ /ylo/ /yhi/ /gray/ recfd --> " << endl;
    psfile <<  "/recfd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    4 index 3 index moveto" << endl;
    psfile <<  "    3 index 3 index lineto" << endl;
    psfile <<  "    3 index 2 index lineto" << endl;
    psfile <<  "    4 index 2 index lineto" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    gsave setgray fill grestore" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "    pop pop pop pop" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Polygon draw op:" << endl;
    psfile <<  "%%   /x1/ /y1/ .. /xn/ /yn/ /n/ pold --> " << endl;
    psfile <<  "/pold" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    3 1 roll moveto" << endl;
    psfile <<  "    1 sub 1 1  3 2 roll { pop lineto } for" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Polygon fill op:" << endl;
    psfile <<  "%%   /x1/ /y1/ .. /xn/ /yn/ /n/ /gray/ polf --> " << endl;
    psfile <<  "/polf" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    setgray" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    3 1 roll moveto" << endl;
    psfile <<  "    1 sub 1 1  3 2 roll { pop lineto } for" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    fill" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Polygon fill and draw op:" << endl;
    psfile <<  "%%   /x1/ /y1/ .. /xn/ /yn/ /n/ /gray/ polfd --> " << endl;
    psfile <<  "/polfd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    4 2 roll moveto" << endl;
    psfile <<  "    exch 1 sub" << endl;
    psfile <<  "    %% -- /x1/ /y1/ .. /x[n-1]/ /y[n-1]/ /gray/ /n-1/" << endl;
    psfile <<  "    1 1  3 2 roll { pop 3 1 roll lineto } for" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    gsave setgray fill grestore" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Circle fill op:" << endl;
    psfile <<  "%%   /x/ /y/ /radius/ /gray/ cirf --> " << endl;
    psfile <<  "/cirf" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    setgray" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "      0 360 arc" << endl;
    psfile <<  "      closepath" << endl;
    psfile <<  "    fill" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Circle draw op:" << endl;
    psfile <<  "%%   /x/ /y/ /radius/ cird --> " << endl;
    psfile <<  "/cird" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "      0 360 arc" << endl;
    psfile <<  "      closepath" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Circle fill and draw op:" << endl;
    psfile <<  "%%   /x/ /y/ /radius/ /gray/ cirfd --> " << endl;
    psfile <<  "/cirfd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    4 1 roll" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "      0 360 arc" << endl;
    psfile <<  "      closepath" << endl;
    psfile <<  "      gsave setgray fill grestore" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Lune fill and draw op:" << endl;
    psfile <<  "%%   /xc/ /yc/ /rad/ /tilt/ /gray/ lunefd --> " << endl;
    psfile <<  "/lunefd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    5 1 roll" << endl;
    psfile <<  "    %% --- gray, xc, yc, rad, tilt" << endl;
    psfile <<  "    4 2 roll" << endl;
    psfile <<  "    %% --- gray, rad, tilt, xc, yc" << endl;
    psfile <<  "    translate" << endl;
    psfile <<  "    %% --- gray, rad, tilt" << endl;
    psfile <<  "    rotate" << endl;
    psfile <<  "    %% --- gray, rad " << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "      dup" << endl;
    psfile <<  "      %% --- gray, rad, rad" << endl;
    psfile <<  "      dup neg 0" << endl;
    psfile <<  "      %% --- gray, rad, rad, -rad, 0" << endl;
    psfile <<  "      3 2 roll 2 mul" << endl;
    psfile <<  "      %% --- gray, rad, -rad, 0, 2*rad" << endl;
    psfile <<  "      -60 60 arc" << endl;
    psfile <<  "      %% --- gray, rad" << endl;
    psfile <<  "      dup 0" << endl;
    psfile <<  "      %% --- gray, rad, rad, 0" << endl;
    psfile <<  "      3 2 roll 2 mul" << endl;
    psfile <<  "      %% --- gray, rad, 0, 2*rad" << endl;
    psfile <<  "      120 240 arc" << endl;
    psfile <<  "      closepath" << endl;
    psfile <<  "      gsave setgray fill grestore" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Cell fill op:" << endl;
    psfile <<  "%%   /xi/ /yi/ celf --> " << endl;
    psfile <<  "/celf" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  3 1 roll " << endl;
    psfile <<  "  exch dup " << endl;
    psfile <<  "  xstep mul xmin add exch 1 add xstep mul xmin add" << endl;
    psfile <<  "  3 2 roll dup" << endl;
    psfile <<  "  ystep mul ymin add exch 1 add ystep mul ymin add" << endl;
    psfile <<  "  5 4 roll " << endl;
    psfile <<  "  recf" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Triangle fill op:" << endl;
    psfile <<  "%%   /xa/ /ya/ /xb/ /yb/ /xc/ /yc/ /gray/ trif --> " << endl;
    psfile <<  "/trif" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    setgray" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    moveto" << endl;
    psfile <<  "    lineto" << endl;
    psfile <<  "    lineto" << endl;
    psfile <<  "    closepath" << endl;
    psfile <<  "    fill" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Segment draw op:" << endl;
    psfile <<  "%%   /xa/ /ya/ /xb/ /yb/ segd --> " << endl;
    psfile <<  "/segd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "    moveto" << endl;
    psfile <<  "    lineto" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Curve draw op:" << endl;
    psfile <<  "%%   /xa/ /ya/  /xb/ /yb/  /xc/ /yc/  /xd/ /yd/ arcd --> " << endl;
    psfile <<  "/arcd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    newpath" << endl;
    psfile <<  "      8 -2 roll moveto curveto" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Draw an X-value grid line:" << endl;
    psfile <<  "%%   /x/ xgrd --> " << endl;
    psfile <<  "/xgrd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "  newpath" << endl;
    psfile <<  "    dup ymin moveto" << endl;
    psfile <<  "    ymax lineto" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Draw an Y-value grid line:" << endl;
    psfile <<  "%%   /y/ ygrd --> " << endl;
    psfile <<  "/ygrd" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "  newpath" << endl;
    psfile <<  "    dup xmin exch moveto" << endl;
    psfile <<  "    xmax exch lineto" << endl;
    psfile <<  "    stroke" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Label printing op:" << endl;
    psfile <<  "%%   /str/ /xa/ /ya/ /xc/ /yc/ lbsh --> " << endl;
    psfile <<  "/lbsh" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  labelfont setfont" << endl;
    psfile <<  "  newpath moveto" << endl;
    psfile <<  "    %% --- str, xa, ya" << endl;
    psfile <<  "  gsave 2 index false charpath flattenpath pathbbox grestore" << endl;
    psfile <<  "    %% --- str, xa, ya, lox, loy, hix, hiy" << endl;
    psfile <<  "  3 index 3 index currentpoint " << endl;
    psfile <<  "    %% --- str, xa, ya, lox, loy, hix, hiy, lox, loy, cx, cy" << endl;
    psfile <<  "  exch 4 1 roll exch sub" << endl;
    psfile <<  "  3 1 roll sub exch" << endl;
    psfile <<  "    %% --- str, xa, ya, lox, loy, hix, hiy, cx-lox, cy-loy" << endl;
    psfile <<  "  rmoveto" << endl;
    psfile <<  "    %% --- str, xa, ya, lox, loy, hix, hiy" << endl;
    psfile <<  "  exch 4 1 roll exch sub " << endl;
    psfile <<  "  3 1 roll sub exch" << endl;
    psfile <<  "    %% --- str, xa, ya, dx, dy" << endl;
    psfile <<  "  exch 4 1 roll mul -1 mul" << endl;
    psfile <<  "  3 1 roll mul -1 mul exch" << endl;
    psfile <<  "    %% --- str, -dx*xa, -dy*ya" << endl;
    psfile <<  "  rmoveto" << endl;
    psfile <<  "    %% --- str" << endl;
    psfile <<  "  show" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile.flush();
  }

void PSFile::define_caption_procs()
  {
    psfile <<  "%% op to move to new caption line:" << endl;
    psfile <<  "%%   nl --> " << endl;
    psfile <<  "/nl" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  /ytext ytext dytext sub def" << endl;
    psfile <<  "  xtext ytext moveto" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% op to print string at CP without clipping:" << endl;
    psfile <<  "%%   /s/ shw --> " << endl;
    psfile <<  "/shw" << endl;
    psfile <<  "{" << endl;
    psfile <<  "  gsave" << endl;
    psfile <<  "    initclip" << endl;
    psfile <<  "    captionfont setfont show" << endl;
    psfile <<  "  grestore" << endl;
    psfile <<  "} def" << endl;
    psfile <<  "" << endl;

    psfile.flush();
  }

void PSFile::setup_page_state(int xn, int yn)
  {
    double fxn = xn;
    double fyn = yn;

    psfile <<  "%% Round joints and caps:" << endl;
    psfile <<  "1 setlinecap 1 setlinejoin" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Black thin lines:" << endl;
    psfile <<  "0 setlinewidth 0 setgray [ ] 0 setdash" << endl;
    psfile <<  "" << endl;

    psfile <<  "/xmin "<< hmin <<" def   %% min plottable x" << endl;
    psfile <<  "/xmax "<< hmax <<" def   %% max plottable x" << endl;
    psfile <<  "/xn "<< xn <<" def     %% grid cells along x axis" << endl;
    psfile <<  "/xstep "<< ((hmax-hmin)/fxn) <<" def  %% x-size of grid cell" << endl;

    psfile <<  "/ymin "<< vmin <<" def   %% min plottable y" << endl;
    psfile <<  "/ymax "<< vmax<<" def   %% max plottable y" << endl;
    psfile <<  "/yn "<< yn<<" def     %% grid cells along y axis" << endl;
    psfile <<  "/ystep "<< ((vmax-vmin)/fyn)<<" def  %% y-size of grid cell" << endl;

    psfile <<  "%% Units of measure:" << endl;
    psfile <<  "/pt 1.0 def" << endl;
    psfile <<  "/in pt 72.0 mul def " << endl;
    psfile <<  "/mm pt 72.0 25.4 div mul def" << endl;
    psfile <<  "" << endl;
    
    set_label_font ("Courier", 8.0);    

    psfile <<  "%% Set clipping path to boundary of plot area:" << endl;
    psfile <<  "newpath" << endl;
    psfile <<  "  xmin ymin moveto" << endl;
    psfile <<  "  xmax ymin lineto" << endl;
    psfile <<  "  xmax ymax lineto" << endl;
    psfile <<  "  xmin ymax lineto" << endl;
    psfile <<  "  xmin ymin lineto" << endl;
    psfile <<  "clip" << endl;
    psfile <<  "" << endl;

    psfile.flush();
  }
  
void PSFile::setup_caption_data()
  {
    psfile <<  "%% Caption text cursor:" << endl;
    psfile <<  "/xtext xmin def" << endl;
    psfile <<  "/ytext ymin def" << endl;
    psfile <<  "/dytext 10 pt mul def" << endl;
    psfile <<  "" << endl;

    psfile <<  "%% Caption font setup:" << endl;
    psfile <<  "/captionfont" << endl;
    psfile <<  "  /Courier findfont" << endl;
    psfile <<  "  dytext scalefont" << endl;
    psfile <<  "def" << endl;
    psfile <<  "" << endl;
  }
  
void PSFile::aux_end_page()
  {
    psfile <<  "savedstate restore" << endl;
    psfile <<  "%% Now we are back to the standard coord system." << endl;
    psfile <<  "" << endl;
    psfile <<  "end %% $maindict" << endl;
    psfile <<  "" << endl;
    psfile.flush();
  }

void PSFile::begin_section(char *title)
  {
    psfile <<  "%% "<< title << endl;
    psfile.flush();
  }

void PSFile::end_section()
  {
    psfile <<  "" << endl;
    psfile.flush();
  }

void PSFile::add_caption(char *txt)
  {
    psfile <<  "nl ";
    put_text(txt, ") shw\nnl (");
    psfile <<  " shw" << endl;
    psfile.flush();
  }
  
void PSFile::set_label_font(char *font, float size)
  {
    psfile <<  "%% Label font setup:" << endl;
    psfile <<  "/labelfont" << endl;
    psfile <<  "  /"<< font<<" findfont "<< size<< " pt mul scalefont" << endl;
    psfile <<  "def" << endl;
    psfile <<  "" << endl;
    psfile.flush();
  }
  
void PSFile::put_label(
    char *text, 
    double x, double y, 
    float xalign, float yalign
  )
  {
    double psx = hmin + xscale * (x - xmin);
    double psy = vmin + yscale * (y - ymin);
    put_text(text, "\\267");
    psfile <<  " ";
    psfile << xalign<<' '<<yalign<<' '<<psx<<' '<<psy << " lbsh"<<endl;
    psfile.flush();
  }

void PSFile::draw_frame ()
  {
    begin_section("Draw frame around plot area");
    psfile <<  "gsave" << endl;
    psfile <<  "%% Assumes xmax, xmin, ymax, ymin are defined." << endl;
    psfile <<  "  initclip" << endl;
    psfile <<  "  newpath" << endl;
    psfile <<  "  xmin ymin moveto" << endl;
    psfile <<  "  xmax ymin lineto" << endl;
    psfile <<  "  xmax ymax lineto" << endl;
    psfile <<  "  xmin ymax lineto" << endl;
    psfile <<  "  xmin ymin lineto" << endl;
    psfile <<  "  closepath stroke" << endl;
    psfile <<  "grestore" << endl;
    end_section();
  }

void PSFile::set_pen(

    double gray,
    double width,
    double dashlength,
    double dashspace
  )
  {
    psfile << gray << "setgray" << endl;
    psfile <<  "mm "<<width<<" mul setlinewidth" << endl;
    if ((dashlength == 0.0) | (dashspace == 0.0))
      { 
      	psfile <<  "[ ] 0 setdash" << endl; 
      }
    else
      { 
      	psfile <<  "[ "<<dashlength<<" mm mul "<<dashspace<<" mm mul ] 0 setdash"<<endl;
      }
    psfile <<  "" << endl;
    psfile.flush();
  }

void PSFile::draw_segment(

    double xa, double ya,
    double xb, double yb
  )
  {
    double psxa = hmin + xscale * (xa - xmin);
    double psya = vmin + yscale * (ya - ymin);
    double psxb = hmin + xscale * (xb - xmin);
    double psyb = vmin + yscale * (yb - ymin);
    psfile << psxa<<' '<<psya<<' '<<psxb<<' '<<psyb << "segd" << endl;
    psfile.flush();
  }

void PSFile::draw_curve(

    double xa, double ya,
    double xb, double yb,
    double xc, double yc,
    double xd, double yd
  )
  {
    double psxa = hmin + xscale * (xa - xmin);
    double psya = vmin + yscale * (ya - ymin);
    double psxb = hmin + xscale * (xb - xmin);
    double psyb = vmin + yscale * (yb - ymin);
    double psxc = hmin + xscale * (xc - xmin);
    double psyc = vmin + yscale * (yc - ymin);
    double psxd = hmin + xscale * (xd - xmin);
    double psyd = vmin + yscale * (yd - ymin);
    psfile << psxa<<' '<<psya<<' '<<psxb<<' '<<psyb<<' '<<psxc<<' '<<psyc<<' '<<psxd<<' '<<psyd<<" arcd"<<endl;
    psfile.flush();
  }
  
void PSFile::aux_rectangle(

    double xlo, double xhi,
    double ylo, double yhi,
    double gray,
    char *op
  )
  {
    double psxlo = hmin + xscale * (xlo - xmin);
    double psxhi = hmin + xscale * (xhi - xmin);
    double psylo = vmin + yscale * (ylo - ymin);
    double psyhi = vmin + yscale * (yhi - ymin);
    psfile << psxlo<<' '<<psxhi<<' '<<psylo<<' '<<psyhi;
    if (gray >= 0.0) psfile <<  ' '<< gray;
    psfile <<  " "<< op << endl;
    psfile.flush();
  }

void PSFile::draw_rectangle(

    double xlo, double xhi,
    double ylo, double yhi
  )
  {
    aux_rectangle(xlo, xhi, ylo, yhi, -1.0, "recd");
  }

void PSFile::fill_rectangle(

    double xlo, double xhi,
    double ylo, double yhi,
    double gray
  )
  {
    aux_rectangle(xlo, xhi, ylo, yhi, gray, "recf");
  }

void PSFile::fill_and_draw_rectangle(

    double xlo, double xhi,
    double ylo, double yhi,
    double gray
  )
  {
    aux_rectangle(xlo, xhi, ylo, yhi, gray, "recfd");
  }

void PSFile::aux_polygon(
    double x[], double y[],
    int npoints,
    double gray,
    char *op
  )
  {
    int i;
    if (npoints>6) psfile <<  "" << endl;
    for (i=0; i<npoints; i++)
      {
        double psxi = hmin + xscale * (x[i] - xmin);
        double psyi = vmin + yscale * (y[i] - ymin);
        if ((i % 6) == 0) 
          { if (i>0) psfile <<  "" << endl; }
        else
          { psfile <<  "  "; }
        psfile <<  psxi<<' '<<psyi;
      }
    if (npoints>6) psfile <<  "" << endl;
    psfile <<  "  "<< npoints<<"";
    if (gray >= 0.0) psfile <<  ' ' << gray;
    psfile <<  " "<< op << endl;
    psfile.flush();
  }

void PSFile::draw_polygon(
    double x[], double y[],
    int npoints
  )
  {
    aux_polygon(x, y, npoints, -1.0, "pold");
  }
    
void PSFile::fill_polygon(
    double x[], double y[],
    int npoints,
    double gray
  )
  {
    aux_polygon(x, y, npoints, gray, "polf");
  }
    
void PSFile::fill_and_draw_polygon(
    double x[], double y[],
    int npoints,
    double gray
  )
  {
    aux_polygon(x, y, npoints, gray, "polfd");
  }
    
void PSFile::aux_circle(
    double xc, double yc, double radius,
    double gray,
    char *op
  )
  {
    double psxc = hmin + xscale * (xc - xmin);
    double psyc = vmin + yscale * (yc - ymin);
    double psradius = yscale * radius;
    psfile <<  psxc<<' '<<psyc<<' '<<psradius; 
    if (gray >= 0.0) psfile <<  ' ' << gray;
    psfile <<  " "<< op <<endl;
    psfile.flush();
  }

void PSFile::fill_circle(
    double xc, double yc, double radius,
    double gray
  )
  {
    aux_circle(xc, yc, radius, gray, "cirf");
  }
  
void PSFile::draw_circle(
    double xc, double yc, double radius
  )
  {
    aux_circle(xc, yc, radius, -1.0, "cird");
  }
  
void PSFile::fill_and_draw_circle(
    double xc, double yc, double radius,
    double gray
  )
  {
    aux_circle(xc, yc, radius, gray, "cirfd");
  }
  
void PSFile::aux_lune(
    double xc, double yc, double radius, double tilt,
    double gray,
    char *op
  )
  {
    double psxc = hmin + xscale * (xc - xmin);
    double psyc = vmin + yscale * (yc - ymin);
    double psradius = yscale * radius;
    double pstilt = tilt * 180.0 / 3.1415926;
    psfile << psxc<<' '<< psyc<<' '<<psradius<<' '<<pstilt; 
    if (gray >= 0.0) psfile <<  ' ' <<  gray;
    psfile <<  " "<< op <<endl;
    psfile.flush();
  }

void PSFile::fill_and_draw_lune(
    double xc, double yc, double radius, double tilt,
    double gray
  )
  {
    aux_lune(xc, yc, radius, tilt, gray, "lunefd");
  }
  
void PSFile::fill_triangle(
    double xa, double ya,
    double xb, double yb,
    double xc, double yc,
    double gray
  )
  {
    double psxa = hmin + xscale * (xa - xmin);
    double psya = vmin + yscale * (ya - ymin);
    double psxb = hmin + xscale * (xb - xmin);
    double psyb = vmin + yscale * (yb - ymin);
    double psxc = hmin + xscale * (xc - xmin);
    double psyc = vmin + yscale * (yc - ymin);
    psfile << psxa<<' '<<psya<<' '<<psxb<<' '<<psyb<<' '<<psxc<<' '<<psyc<<' '<<gray << " trif" << endl;
    psfile.flush();
  }

void PSFile::fill_grid_cell(int xi, int yi, double gray)
  {
    psfile <<  xi<<' '<<yi<<' '<<gray<<" celf"<<endl;
    psfile.flush();
  }

void PSFile::draw_coord_line (char axis, double coord)
  {
    double pscoord;
    if (axis == 'x')
      { pscoord = hmin + xscale * (coord - xmin); }
    else if (axis == 'y')
      { pscoord = vmin + yscale * (coord - ymin); }
    else
      { 
      	return;
      }
    psfile <<  pscoord<< axis<< "grd"<<endl;
  }

void PSFile::draw_grid_lines()
  {
    psfile <<  "%% Grid lines:" << endl;
    psfile <<  "gsave" << endl;
    psfile <<  "  initclip" << endl;
    psfile <<  "  0 1 xn {" << endl;
    psfile <<  "    xstep mul xmin add xgrd" << endl;
    psfile <<  "  } for" << endl;
    psfile <<  "  0 1 yn {" << endl;
    psfile <<  "    ystep mul ymin add ygrd" << endl;
    psfile <<  "  } for" << endl;
    psfile <<  "grestore" << endl;
    psfile <<  "" << endl;
    psfile.flush();
  }

void PSFile::put_text(char *text, char *newline)
  {
    char *p;
    psfile<<'(';
    for (p = text; *p != 0; p++)
      {
        if (*p == '\n')
          { psfile << ""<< newline <<"";}
        else if (*p == '(')
          { psfile<<'\\'; psfile<<'('; }
        else if (*p == ')')
          { psfile<<'\\'; psfile<<')'; }
        else if (*p == '\t')
          { psfile<<' '; psfile<<' '; }
        else if (*p == '\\')
          { psfile<<'\\'; psfile<<'\\'; }
        else if ((*p < ' ') || (*p > '~'))
          { psfile <<  '\\'<< *p << 'o'; }
        else
          { psfile<<*p; }
      }
    psfile <<  ')';
  }

PSFile& PSFile::operator<<(const Model& m)
{
	psfile << m;
	return (*this);
}


}
