/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Utils.h"
#include <string>
#include <vector>


namespace Modelib {

namespace Utils
{
	using namespace std;
	
	void Tokenize(const string& str, vector<string>& tokens, const string& delimiters)
	{
		// Skip delimiters at beginning.
		string::size_type lastPos = str.find_first_not_of(delimiters, 0);
		// Find first "non-delimiter".
		string::size_type pos     = str.find_first_of(delimiters, lastPos);
	
		while (string::npos != pos || string::npos != lastPos)
		{
			// Found a token, add it to the vector.
			tokens.push_back(str.substr(lastPos, pos - lastPos));
			// Skip delimiters.  Note the "not_of"
			lastPos = str.find_first_not_of(delimiters, pos);
			// Find next "non-delimiter"
			pos = str.find_first_of(delimiters, lastPos);
		}
	}
	

	/**
	   AvoidSpaces permet d'liminer tous les espaces de la chaine de caractre
	   passe eb argument.

	   @param s string de la STL qui contient la chaine de caractre
	   @return Le string sans espaces
	*/
	string AvoidSpaces(string& s)
	{
		string t;
		for(string::const_iterator iter = s.begin();iter!=s.end();++iter)
			if (*iter != ' ' && *iter != '\t')
				t += *iter;
		return t;
	}

	/**
		Remplacer une sous-string par un string.\n
		Ce remplacement s'effectue sur tout le string

		@param where	Le string o on souhaite remplacer le mot
		@param what		Ce que l'on veut remplacer
		@param by		Le string qui va remplacer
	*/
	void Replace(string& where, const string& what, const string& by)
	{
		for (string::size_type 	i  = where.find(what);
								i != string::npos;
								i  = where.find(what, i + by.size()))
			where.replace(i, what.size(), by);
	}

	/**
		Convertir un string en compatible HTML

		@param str Le string a interprter
		@result string = Le string rsultant
	*/
	string toHTML(const string& str)
	{
		string ret(str);
		Replace(ret, "<", "&lt;");
		Replace(ret, ">", "&gt;");
		return ret;
	}
	
	/**
		Convertir un string en compatible LaTeXs

		@param str Le string a interprter
		@result string = Le string rsultant
	*/
	string toLATEX(const string& str)
	{
		string ret(str);
		Replace(ret, "<=", "\\leq");
		Replace(ret, ">=", "\\geq");
		Replace(ret, "!=", "\\neq");
		Replace(ret, "==", "=");
		Replace(ret, "$", "");
		return ret;
	}

}

}
